@@ -109,6 +109,54 @@ class I18nBackendFallbacksTranslateTest < I18n::TestCase
   end
 end
 
+# See Issue #534
+class I18nBackendFallbacksLocalizeTestWithDefaultLocale < I18n::TestCase
+  class Backend < I18n::Backend::Simple
+    include I18n::Backend::Fallbacks
+  end
+
+  def setup
+    super
+    I18n.backend = Backend.new
+    I18n.enforce_available_locales = false
+    I18n.fallbacks = [I18n.default_locale]
+    store_translations(:en, time: { formats: { fallback: 'en fallback' } })
+  end
+
+  test "falls back to default locale - Issue #534" do
+    assert_equal 'en fallback', I18n.l(Time.now, format: :fallback, locale: "un-supported")
+  end
+end
+
+# See Issue #536
+class I18nBackendFallbacksWithCustomClass < I18n::TestCase
+  class BackendWithFallbacks < I18n::Backend::Simple
+    include I18n::Backend::Fallbacks
+  end
+
+  # Quacks like a fallback class
+  class MyDefaultFallback
+    def [](key)
+     [:my_language]
+    end
+  end
+
+  def setup
+    super
+    I18n.backend = BackendWithFallbacks.new
+    I18n.enforce_available_locales = false
+    I18n.fallbacks = MyDefaultFallback.new
+    store_translations(:my_language, foo: 'customer foo')
+    store_translations(:en, foo: 'english foo')
+  end
+
+  test "can use a default fallback object that doesn't inherit from I18n::Locale::Fallbacks" do
+    assert_equal 'customer foo', I18n.t(:foo, locale: :en)
+    assert_equal 'customer foo', I18n.t(:foo, locale: :nothing)
+  end
+end
+
+
 class I18nBackendFallbacksLocalizeTest < I18n::TestCase
   class Backend < I18n::Backend::Simple
     include I18n::Backend::Fallbacks
@@ -158,6 +206,14 @@ class I18nBackendFallbacksWithChainTest < I18n::TestCase
     assert_equal 'FOO', I18n.t(:foo, :locale => :'de-DE')
   end
 
+  test "exists? falls back from de-DE to de given a key missing from the given locale" do
+    assert_equal true, I18n.exists?(:foo, :locale => :'de-DE')
+  end
+
+  test "exists? should return false when fallback disabled given a key missing from the given locale" do
+    assert_equal false, I18n.exists?(:foo, :locale => :'de-DE', fallback: false)
+  end
+
   test "falls back from de-DE to de when there is no translation for de-DE available when using arrays, too" do
     assert_equal ['FOO', 'FOO'], I18n.t([:foo, :foo], :locale => :'de-DE')
   end
@@ -212,4 +268,43 @@ class I18nBackendFallbacksExistsTest < I18n::TestCase
     assert_equal false, I18n.exists?(:baz, :de)
     assert_equal false, I18n.exists?(:bogus, :'de-DE')
   end
+
+  test "exists? should return false when fallback is disabled given a key which is missing from the given locale" do
+    assert_equal true, I18n.exists?(:bar, :'de-DE')
+    assert_equal false, I18n.exists?(:bar, :'de-DE', fallback: false)
+    assert_equal false, I18n.exists?(:bar, :'de-DE-XX', fallback: false)
+  end
+end
+
+class I18nBackendOnFallbackHookTest < I18n::TestCase
+  class Backend < I18n::Backend::Simple
+    include I18n::Backend::Fallbacks
+
+    attr :fallback_collector
+
+    private
+
+      def on_fallback(*args)
+        @fallback_collector ||= []
+        @fallback_collector << args
+      end
+  end
+
+  def setup
+    super
+    I18n.backend = Backend.new
+    I18n.fallbacks = I18n::Locale::Fallbacks.new(de: :en)
+    store_translations(:en, :foo => 'Foo in :en', :bar => 'Bar in :en')
+    store_translations(:de, :bar => 'Bar in :de')
+    store_translations(:"de-DE", :baz => 'Baz in :"de-DE"')
+  end
+
+  test "on_fallback should be called when fallback happens" do
+    assert_equal [:"de-DE", :de, :en], I18n.fallbacks[:"de-DE"]
+    assert_equal 'Baz in :"de-DE"', I18n.t(:baz, locale: :'de-DE')
+    assert_equal 'Bar in :de', I18n.t(:bar, locale: :'de-DE')
+    assert_equal 'Foo in :en', I18n.t(:foo, locale: :'de-DE')
+    assert_equal [:'de-DE', :de, :bar, {}], I18n.backend.fallback_collector[0]
+    assert_equal [:'de-DE', :en, :foo, {}], I18n.backend.fallback_collector[1]
+  end
 end
